<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Estimates_pmf extends CI_Model
{
	function __construct()
	{
		parent::__construct();
		$this->load->library(OTBASE_PATH.'OT_App');
        $this->load->library(OTBASE_PATH.'OT_User');
	}
	
 	public function get_next_number($with_last=false)
    {
    	$next_num = '0001';
    	
    	$this->db->select('estimate_number');
    	$this->db->from('estimate');
    	$this->db->where(array('entity_id' => $this->ot_app->get('entity_id')));
    	$this->db->order_by('estimate_id', 'desc');
    	$this->db->limit(1);
    	$query	= $this->db->get();

    	$res	= $query->row();

    	$return	= array();
    	if ($res)
    	{
    		$pattern	= '/[0-9]+$/';
    		$return['last']	= $res->estimate_number;
    		if (preg_match($pattern, $res->estimate_number,$matches))
    		{
    			$next_num	= str_pad($matches[0]+1, strlen($matches[0]), 0, STR_PAD_LEFT);
				$next_num	= preg_replace($pattern, $next_num, $res->estimate_number);
    		}
    		else
    		{
    			$next_num	= $res->estimate_number.'1';
    		}
    	}

    	$return['next']	= $next_num;
    	if ($with_last)
    	{
    		return $return;
    	}

    	return $next_num;
    }
	
	public function list_query($select, $options = array())
	{
		$select	= ($select)?$select:"et.estimate_id, et.estimate_number, DATE_FORMAT(et.estimate_date, '%Y-%m-%d') as estimate_date, et.status, et.amount, et.currency_code, c.client_id,  concat(c.client_code, c.client_name) AS client_name";
		$this->db->select($select, false);
		$this->db->from('estimate AS et');
		$this->db->join('client AS c', 'et.entity_id=c.entity_id and et.client_id = c.client_id', 'INNER');
		
		$where	= array('et.entity_id' => $this->ot_app->get('entity_id'));
		if (!empty($options['filters']))
		{
			foreach($options['filters'] as $key=>$val) {
				if (is_array($val))
					$this->db->where_in($key,$val);
				else
					$where[$key] = $val;
			}			
		}
		$this->db->where($where);
		
		if (!empty($options['group_by']))
		{
			$this->db->group_by($options['group_by']);
		}

		$order_by = !empty($options['order_by'])?$options['order_by']:'et.estimate_id';
		$this->db->order_by($order_by, 'DESC');

		$from = !empty($options['from'])?$options['from']:0;
		if (!empty($options['count']))
		{
			$this->db->limit($options['count'], $from);
		}

		return $this->db->get();
	}

	public function get_list($select = '', $options = array())
	{
		$options['filters']['et.created_by'] = $this->ot_user->get('user_id');
		$options['filters']['et.status'] 	=	'd';
		
		$query	= $this->list_query($select, $options);

		return $query->result_array();
	}

	public function get_details($estimate_id='')
	{
		if ($estimate_id)
		{
			$this->db->select('ei.*');
			$this->db->from('estimate_items AS ei');
			$this->db->where('ei.estimate_id', $estimate_id);
			$this->db->order_by('ei.serial_no');
			$query	= $this->db->get();

			return $query->result_array();
		}
		return false;
	}
	
	public function get_client_details($client_id)
	{	
		$this->db->select('et.client_contact_id, et.currency_code, et.address, c.address as client_address');
		$this->db->from('client AS c');
		$this->db->join('estimate AS et', 'c.entity_id=et.entity_id and c.client_id = et.client_id', 'left');
		$this->db->where(array('c.entity_id' => $this->ot_app->get('entity_id'), 'c.client_id' => $client_id));
		$this->db->order_by('et.estimate_id', 'desc');
		$this->db->limit(1);
		
		$query	= $this->db->get();
		return $query->row_array();
	}
	
	public function save($data)
	{
		$this->load->library('estimate_table');
		
		$params = array();
		if (!empty($data[$this->estimate_table->tbl_key]))
		{
			$params['where'] = array('entity_id' => $this->ot_app->get('entity_id'), 'estimate_id' => $data[$this->estimate_table->tbl_key], 'created_by' => $this->ot_user->get('user_id'), 'status' => 'd');
		}
		$result = $this->estimate_table->save($data,$params);
		 
		return $result;
	}
	
	public function filter_where($estimate_id)
	{
			$where['estimate_id']	= $estimate_id;
			$where['entity_id']		= $this->ot_app->get('entity_id');
			$where['created_by'] 	= $this->ot_user->get('user_id');
			$where['status']		= 'd';
			return $where;
	}
	public function delete($estimate_id)
	{
		$estimate_id = (int)$estimate_id;
		$est_result	= $this->get_list("et.estimate_number, et.amount, et.currency_code", array('filters' => array('et.estimate_id'=>$estimate_id)));
		if (!$estimate_id)
		{
			return false;
		}

		$where = $this->filter_where($estimate_id);
		
		if ($this->db->delete('estimate', $where)) {
			$this->db->delete('estimate_items', array('estimate_id' => $estimate_id));
		}
		// Insert Estimate activity
		$fullname = $this->ot_user->get('first_name').' '.$this->ot_user->get('last_name').' ';
		$result = $est_result[0];
		
		$currency_symbol = get_currency($result['currency_code']);		
		$act_data = array(
				'invoice_id' 	=> $estimate_id,					
				'a_date'		=> $this->auth->get_timezone_date(),
				'a_mode'		=> 'ED',
				'a_notes'		=> $fullname. ' deleted Estimate #'.$result['estimate_number'].' ('.$currency_symbol.' '.$result['amount'].')'
		);
		if (! invoice_activity($act_data))
			return false;		

		return true;
	}
	
	public function duplicate($estimate_id)
	{
		// get estimate data
		$this->db->select('*');
		$this->db->from('estimate');

		$where = $this->filter_where($estimate_id);

		$this->db->where($where);
		$query	= $this->db->get();
		$result =  $query->row_array();
		
		if ($result) {
			// change status as draft if it's in invoiced status
			if ($result['status']=='i') {
				$result['status'] = 'd';
			}
			$unset_array = array('estimate_id','estimate_number','last_updated_by', 'last_updated_date');
			foreach ($unset_array as $key) unset($result[$key]);
			
			$result['estimate_number'] = $this->get_next_number();
//			$result['estimate_date'] = $this->auth->get_timezone_date();
			// get estimate items
			$details = $this->get_details($estimate_id);
			if ($details)
			{
				foreach ($details as $key=>$row)
				{
					unset($row['estimate_id']);
					$row['estimate_item_id'] = 0;
					$row['serial_no'] = 0;
					$result['details'][$key] = $row;
				}
			}
		}
		$return = array();
		if ($return = $this->save($result))
		{
			$return['estimate_number']	= $result['estimate_number'];
		}
		
		return $return;
	}
}
